@@ -77,6 +77,7 @@ u64 wrap_ticks_to_boot;
 #if defined(CONFIG_X86_64)
 static struct timer_list shared_data_timer;
 struct kuser_shared_data kuser_shared_data;
+/* static void update_user_shared_data_proc(unsigned long data); */
 #endif
 
 WIN_SYMBOL_MAP("KeTickCount", &jiffies)
@@ -90,10 +91,10 @@ DEFINE_PER_CPU(struct irql_info, irql_info);
 #endif
 
 #if defined(CONFIG_X86_64)
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)
-static void update_user_shared_data_proc(unsigned long data)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void update_user_shared_data_proc(struct timer_list *tl)
 #else
-static void update_user_shared_data_proc(struct timer_list *t)
+static void update_user_shared_data_proc(unsigned long data)
 #endif
 {
 	/* timer is supposed to be scheduled every 10ms, but bigger
@@ -410,14 +411,16 @@ static void initialize_object(struct dispatcher_header *dh, enum dh_type type,
 	InitializeListHead(&dh->wait_blocks);
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)
-static void timer_proc(unsigned long data)
-{
-	struct wrap_timer *wrap_timer = (struct wrap_timer *)data;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+static void timer_proc(struct timer_list *tl)
 #else
-static void timer_proc(struct timer_list *t)
+static void timer_proc(unsigned long data)
+#endif
 {
-	struct wrap_timer *wrap_timer = from_timer(wrap_timer, t, timer);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,15,0)
+	struct wrap_timer *wrap_timer = from_timer(wrap_timer, tl, timer);
+#else
+	struct wrap_timer *wrap_timer = (struct wrap_timer *)data;
 #endif
 	struct nt_timer *nt_timer;
 	struct kdpc *kdpc;
@@ -463,8 +466,8 @@ void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)
 	init_timer(&wrap_timer->timer);
-	wrap_timer->timer.data = (unsigned long)wrap_timer;
 	wrap_timer->timer.function = timer_proc;
+	wrap_timer->timer.data = (unsigned long)wrap_timer;
 #else
 	timer_setup(&wrap_timer->timer, timer_proc, 0);
 #endif
@@ -2498,7 +2501,9 @@ struct worker_init_struct {
 
 int ntoskernel_init(void)
 {
-	struct timespec64 now;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0)
+	struct timeval now;
+#endif
 
 	spin_lock_init(&dispatcher_lock);
 	spin_lock_init(&ntoskernel_lock);
@@ -2518,10 +2523,14 @@ int ntoskernel_init(void)
 	INIT_WORK(&ntos_work, ntos_work_worker);
 	wrap_timer_slist.next = NULL;
 
-	ktime_get_real_ts64(&now);
 	wrap_ticks_to_boot = TICKS_1601_TO_1970;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5,0,0)
+	do_gettimeofday(&now);
 	wrap_ticks_to_boot += (u64)now.tv_sec * TICKSPERSEC;
-	wrap_ticks_to_boot += now.tv_nsec / 100;
+	wrap_ticks_to_boot += now.tv_usec * 10;
+#else
+	wrap_ticks_to_boot += ktime_to_us(ktime_get_real()) * 10;
+#endif
 	wrap_ticks_to_boot -= jiffies * TICKSPERJIFFY;
 	TRACE2("%llu", wrap_ticks_to_boot);
 
